"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class AuditLogEntryResourceTypedDict(TypedDict):
    type: str
    r"""The type of the resource."""
    id: str
    r"""The ID of the resource."""
    name: str
    r"""The descriptive name of the resource."""


class AuditLogEntryResource(BaseModel):
    type: str
    r"""The type of the resource."""

    id: str
    r"""The ID of the resource."""

    name: str
    r"""The descriptive name of the resource."""
