"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .forterantifraudoptionsdiscountcoupondiscountamount import (
    ForterAntiFraudOptionsDiscountCouponDiscountAmount,
    ForterAntiFraudOptionsDiscountCouponDiscountAmountTypedDict,
)
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class ForterAntiFraudOptionsDiscountTypedDict(TypedDict):
    coupon_code_used: str
    r"""The coupon code applied to the order."""
    discount_type: str
    r"""The type of discount applied to the order."""
    coupon_discount_amount: NotRequired[
        Nullable[ForterAntiFraudOptionsDiscountCouponDiscountAmountTypedDict]
    ]
    r"""Monetary details of the discount amount."""
    coupon_discount_percent: NotRequired[Nullable[str]]
    r"""The percentage discount applied via the coupon."""


class ForterAntiFraudOptionsDiscount(BaseModel):
    coupon_code_used: str
    r"""The coupon code applied to the order."""

    discount_type: str
    r"""The type of discount applied to the order."""

    coupon_discount_amount: OptionalNullable[
        ForterAntiFraudOptionsDiscountCouponDiscountAmount
    ] = UNSET
    r"""Monetary details of the discount amount."""

    coupon_discount_percent: OptionalNullable[str] = UNSET
    r"""The percentage discount applied via the coupon."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["coupon_discount_amount", "coupon_discount_percent"]
        nullable_fields = ["coupon_discount_amount", "coupon_discount_percent"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
