"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class CheckoutPayoutOptionsTypedDict(TypedDict):
    processing_channel_id: str
    r"""The processing channel to be used for the payment."""
    source_id: str
    r"""The ID of the currency account that will fund the payout."""


class CheckoutPayoutOptions(BaseModel):
    processing_channel_id: str
    r"""The processing channel to be used for the payment."""

    source_id: str
    r"""The ID of the currency account that will fund the payout."""
