"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .digitalwallet import DigitalWallet, DigitalWalletTypedDict
from gr4vy.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class DigitalWalletsTypedDict(TypedDict):
    items: List[DigitalWalletTypedDict]
    r"""A list of items returned for this request."""


class DigitalWallets(BaseModel):
    items: List[DigitalWallet]
    r"""A list of items returned for this request."""
